import argparse
from overrides import overrides

from sacrerouge.commands import Subcommand
from sacrerouge.metrics import autosummeng, bertscore, bewte, meteor, moverscore, python_rouge, rouge, simetrix, sumqe


class SetupMetricSubcommand(Subcommand):
    @overrides
    def add_subparser(self, parser: argparse._SubParsersAction):
        description = 'Setup an evaluation metric'
        self.parser = parser.add_parser('setup-metric', description=description, help=description)
        subparsers = self.parser.add_subparsers()

        subcommands = [
            autosummeng.AutoSummENGSetupSubcommand(),
            bertscore.BertScoreSetupSubcommand(),
            bewte.BEwTESetupSubcommand(),
            meteor.MeteorSetupSubcommand(),
            moverscore.MoverScoreSetupSubcommand(),
            python_rouge.PythonRougeSetupSubcommand(),
            rouge.RougeSetupSubcommand(),
            simetrix.SIMetrixSetupSubcommand(),
            sumqe.SumQESetupSubcommand(),
        ]
        for subcommand in subcommands:
            subcommand.add_subparser(subparsers)

        self.parser.set_defaults(func=self.run)

    @overrides
    def run(self, args):
        if 'subfunc' in dir(args):
            args.subfunc(args)
        else:
            self.parser.print_help()



from collections import defaultdict
from typing import Dict, List

from sacrerouge.data import Metrics, MetricsDict
from sacrerouge.data.types import ReferenceType, SummaryType
from sacrerouge.io import JsonlReader


def load_summaries(file_path: str) -> List[SummaryType]:
    fields = []
    for data in JsonlReader(file_path).read():
        fields.append(data['summary'])
    return fields


def load_references(file_path: str) -> List[ReferenceType]:
    fields = []
    for data in JsonlReader(file_path).read():
        if 'summary' in data:
            fields.append([data['summary']['text']])
        elif 'summaries' in data:
            fields.append([summary['text'] for summary in data['summaries']])
        elif 'reference' in data:
            fields.append([data['reference']['text']])
        elif 'references' in data:
            fields.append([reference['text'] for reference in data['references']])
    return fields


def load_metrics_dicts(file_path: str) -> Dict[str, Dict[str, MetricsDict]]:
    metrics_dicts = defaultdict(dict)
    with JsonlReader(file_path, Metrics) as f:
        for instance in f:
            metrics_dicts[instance.instance_id][instance.summarizer_id] = instance.metrics
    return metrics_dicts


import argparse
from overrides import overrides

from sacrerouge.datasets.chaganty2018 import setup
from sacrerouge.commands import Subcommand


class Chaganty2018Subcommand(Subcommand):
    @overrides
    def add_subparser(self, parser: argparse._SubParsersAction):
        description = 'Setup the Chaganty 2018 dataset'
        self.parser = parser.add_parser('chaganty2018', description=description, help=description)
        self.parser.add_argument(
            'output_dir',
            type=str,
            help='The directory where the data should be written'
        )
        self.parser.set_defaults(subfunc=self.run)

    @overrides
    def run(self, args):
        setup.setup(args.output_dir)



import pytest

from sacrerouge.common.testing import FIXTURES_ROOT
from sacrerouge.common.testing.metric_test_cases import ReferenceBasedMetricTestCase
from sacrerouge.io import JsonlReader
from sacrerouge.metrics import Meteor


class TestMeteor(ReferenceBasedMetricTestCase):
        def test_meteor(self):
            # This is a regression test, not necessarily a test for correctness
            metric = Meteor()
            expected_output = [
                {'METEOR': 0.1669124164573825},
                {'METEOR': 0.1948832291162732},
                {'METEOR': 0.20611797558912198},
                {'METEOR': 0.16514495147439157},
                {'METEOR': 0.17681198359839032},
                {'METEOR': 0.1711431947904745},
                {'METEOR': 0.21743329361081287},
                {'METEOR': 0.21450528119487908},
                {'METEOR': 0.16550404185166837},
                {'METEOR': 0.21164463762707278},
                {'METEOR': 0.20412130512572657},
                {'METEOR': 0.18243523574488876}
            ]
            super().assert_expected_output(metric, expected_output)

        def test_bewte_order_invariant(self):
            metric = Meteor()
            self.assert_order_invariant(metric)

        def test_chen2018(self):
            """
            Tests to ensure that Meteor returns the expected score on the
            Chen 2018 data subset. I ran Meteor on the full data (~11k examples)
            which takes too long to run for a unit test. After confirming the numbers
            are the same as what is reported in the paper, I ran the code on just
            the subset, and this test ensures those numbers are returned.
            """
            gold_file_path = f'{FIXTURES_ROOT}/data/chen2018/gold.jsonl'
            model_file_path = f'{FIXTURES_ROOT}/data/chen2018/model.jsonl'

            gold = JsonlReader(gold_file_path).read()
            model = JsonlReader(model_file_path).read()

            gold = [[summary['summary']] for summary in gold]
            model = [summary['summary'] for summary in model]

            meteor = Meteor()
            score, _ = meteor.evaluate(model, gold)
            assert score['METEOR'] == pytest.approx(0.1828372, abs=1e-7)



import argparse

from sacrerouge.commands import correlate, evaluate, metric_command, score, setup_dataset, setup_metric


def main():
    parser = argparse.ArgumentParser()
    subparsers = parser.add_subparsers()

    subcommands = [
        correlate.CorrelateSubcommand(),
        evaluate.EvaluateSubcommand(),
        score.ScoreSubcommand(),
        setup_dataset.SetupDatasetSubcommand(),
        setup_metric.SetupMetricSubcommand()
    ]
    for subcommand in subcommands:
        subcommand.add_subparser(subparsers)

    # Add a command for each individual metric
    metric_command.add_metric_subcommands(subparsers)

    args = parser.parse_args()
    if 'func' in dir(args):
        args.func(args)
    else:
        parser.print_help()


if __name__ == '__main__':
    main()



import argparse
from overrides import overrides

from sacrerouge.commands import Subcommand
from sacrerouge.datasets import chaganty2018
from sacrerouge.datasets.duc_tac import duc2001, duc2002, duc2003, duc2004, duc2005, duc2006, duc2007, tac2008, tac2009, tac2010, tac2011
from sacrerouge.datasets.multiling import multiling2011, multiling2013, multiling2015, multiling2017


class SetupDatasetSubcommand(Subcommand):
    @overrides
    def add_subparser(self, parser: argparse._SubParsersAction):
        description = 'Setup a dataset'
        self.parser = parser.add_parser('setup-dataset', description=description, help=description)
        subparsers = self.parser.add_subparsers()

        subcommands = [
            chaganty2018.Chaganty2018Subcommand(),
            duc2001.DUC2001Subcommand(),
            duc2002.DUC2002Subcommand(),
            duc2003.DUC2003Subcommand(),
            duc2004.DUC2004Subcommand(),
            duc2005.DUC2005Subcommand(),
            duc2006.DUC2006Subcommand(),
            duc2007.DUC2007Subcommand(),
            multiling2011.MultiLing2011Subcommand(),
            multiling2013.MultiLing2013Subcommand(),
            multiling2015.MultiLing2015Subcommand(),
            multiling2017.MultiLing2017Subcommand(),
            tac2008.TAC2008Subcommand(),
            tac2009.TAC2009Subcommand(),
            tac2010.TAC2010Subcommand(),
            tac2011.TAC2011Subcommand(),
        ]
        for subcommand in subcommands:
            subcommand.add_subparser(subparsers)

        self.parser.set_defaults(func=self.run)

    @overrides
    def run(self, args):
        if 'subfunc' in dir(args):
            args.subfunc(args)
        else:
            self.parser.print_help()



import setuptools

setuptools.setup(
    name='sacrerouge',
    version='0.0.5',
    author='Daniel Deutsch',
    description='An open-source library for summarization evaluation metrics',
    url='https://github.com/danieldeutsch/sacrerouge',
    packages=setuptools.find_packages(),
    entry_points={'console_scripts': ['sacrerouge=sacrerouge.__main__:main']},
    python_requires='>=3.6',
    install_requires=[
        'googledrivedownloader',
        'jsons',
        'lxml',
        'nltk',
        'numpy',
        'overrides',
        'pytest',
        'requests',
        'scipy'
    ]
)



import argparse


class Subcommand(object):
    def add_subparser(self, parser: argparse._SubParsersAction) -> None:
        raise NotImplementedError

    def run(self, args):
        raise NotImplementedError



import argparse
import itertools
import json
import logging
import numpy as np
import os
import warnings
from collections import defaultdict
from overrides import overrides
from scipy.stats import kendalltau, pearsonr, spearmanr
from typing import Any, Dict, List, Union

from sacrerouge.commands import Subcommand
from sacrerouge.common.logging import prepare_global_logging
from sacrerouge.data import Metrics, MetricsDict
from sacrerouge.io import JsonlReader

logger = logging.getLogger(__name__)


def load_metrics(metrics_files: List[str]) -> List[Metrics]:
    logger.info(f'Loading metrics from {metrics_files}')
    metrics_dicts = defaultdict(dict)
    for metrics_file in metrics_files:
        with JsonlReader(metrics_file, Metrics) as f:
            for metrics in f:
                if metrics.summarizer_id not in metrics_dicts[metrics.instance_id]:
                    metrics_dicts[metrics.instance_id][metrics.summarizer_id] = metrics
                else:
                    metrics_dicts[metrics.instance_id][metrics.summarizer_id].merge(metrics)

    metrics_list = []
    for metrics_dict in metrics_dicts.values():
        metrics_list.extend(list(metrics_dict.values()))
    logger.info(f'Loaded {len(metrics_list)} instances')
    return metrics_list


def filter_metrics(metrics_list: List[Metrics], summarizer_type: str, metric1: str, metric2: str):
    logger.info(f'Filtering instances to summarizer type "{summarizer_type}" and metrics "{metric1}" and "{metric2}"')

    filtered = []
    incorrect_type = 0
    missing_metric1 = 0
    missing_metric2 = 0
    for metrics in metrics_list:
        if summarizer_type == 'all' or summarizer_type == metrics.summarizer_type:
            if metric1 not in metrics.metrics:
                missing_metric1 += 1
            elif metric2 not in metrics.metrics:
                missing_metric2 += 1
            else:
                filtered.append(metrics)
        else:
            incorrect_type += 1

    if incorrect_type > 0:
        logger.info(f'Filtered {incorrect_type} instances that were not summarizer type "{summarizer_type}"')
    if missing_metric1 > 0:
        logger.info(f'Filtered {missing_metric1} instances that did not have metric "{metric1}"')
    if missing_metric2 > 0:
        logger.info(f'Filtered {missing_metric2} instances that did not have metric "{metric2}"')
    logger.info(f'{len(filtered)} instances remain after filtering')

    return filtered


def aggregate_metrics(metrics_list: List[Metrics]) -> Dict[str, MetricsDict]:
    # The instances must be sorted by the key in order to use itertools.groupby
    metrics_list = sorted(metrics_list, key=lambda metrics: metrics.summarizer_id)
    key_to_metrics = {}
    for key, group in itertools.groupby(metrics_list, lambda metrics: metrics.summarizer_id):
        group_metrics = [member.metrics for member in group]
        key_to_metrics[key] = sum(group_metrics) / len(group_metrics)
    return key_to_metrics


def compute_summary_level_correlations(metrics_list: List[Dict[str, Any]],
                                       metric1: str,
                                       metric2: str) -> Dict[str, float]:
    pearsons = []
    spearmans = []
    kendalls = []
    num_nan = 0
    metrics_list = sorted(metrics_list, key=lambda metrics: metrics.instance_id)
    for _, group in itertools.groupby(metrics_list, key=lambda metrics: metrics.instance_id):
        group = list(group)
        values1 = [member.metrics[metric1] for member in group]
        values2 = [member.metrics[metric2] for member in group]

        with warnings.catch_warnings():
            warnings.filterwarnings('ignore')
            r, _ = pearsonr(values1, values2)
            rho, _ = spearmanr(values1, values2)
            tau, _ = kendalltau(values1, values2)

            if any(np.isnan([r, rho, tau])):
                num_nan += 1
            else:
                pearsons.append(r)
                spearmans.append(rho)
                kendalls.append(tau)

    if num_nan > 0:
        logger.warning(f'Skipped {num_nan} summary-level correlations because they were NaN')

    num_valid = len(pearsons)
    if num_valid > 0:
        pearson = sum(pearsons) / len(pearsons)
        spearman = sum(spearmans) / len(spearmans)
        kendall = sum(kendalls) / len(kendalls)
    else:
        pearson, spearman, kendall = 0, 0, 0

    return {
        'pearson': {
            'r': pearson
        },
        'spearman': {
            'rho': spearman
        },
        'kendall': {
            'tau': kendall
        },
        'num_summary_groups': num_valid
    }


def compute_system_level_correlations(metrics_list: List[Dict[str, Any]],
                                      metric1: str,
                                      metric2: str) -> Dict[str, float]:
    metrics_list = list(aggregate_metrics(metrics_list).values())

    values1 = [metrics[metric1] for metrics in metrics_list]
    values2 = [metrics[metric2] for metrics in metrics_list]

    r, r_pvalue = pearsonr(values1, values2)
    rho, rho_pvalue = spearmanr(values1, values2)
    tau, tau_pvalue = kendalltau(values1, values2)
    num_summarizers = len(metrics_list)

    return {
        'pearson': {
            'r': r,
            'p_value': r_pvalue
        },
        'spearman': {
            'rho': rho,
            'p_value': rho_pvalue
        },
        'kendall': {
            'tau': tau,
            'p_value': tau_pvalue
        },
        'num_summarizers': num_summarizers
    }


def compute_global_correlations(metrics_list: List[Dict[str, Any]],
                                metric1: str,
                                metric2: str) -> Dict[str, float]:
    values1 = [metrics.metrics[metric1] for metrics in metrics_list]
    values2 = [metrics.metrics[metric2] for metrics in metrics_list]

    r, r_pvalue = pearsonr(values1, values2)
    rho, rho_pvalue = spearmanr(values1, values2)
    tau, tau_pvalue = kendalltau(values1, values2)
    num_summaries = len(metrics_list)

    return {
        'pearson': {
            'r': r,
            'p_value': r_pvalue
        },
        'spearman': {
            'rho': rho,
            'p_value': rho_pvalue
        },
        'kendall': {
            'tau': tau,
            'p_value': tau_pvalue
        },
        'num_summaries': num_summaries
    }


def compute_correlation(metrics_jsonl_files: Union[str, List[str]],
                        metric1: str,
                        metric2: str,
                        summarizer_type: str):
    if isinstance(metrics_jsonl_files, str):
        metrics_jsonl_files = [metrics_jsonl_files]

    metrics_list = load_metrics(metrics_jsonl_files)
    for metrics in metrics_list:
        metrics.flatten_keys()

    metrics_list = filter_metrics(metrics_list, summarizer_type, metric1, metric2)
    for metrics in metrics_list:
        metrics.select_metrics([metric1, metric2])
        metrics.average_values()

    summary_level = compute_summary_level_correlations(metrics_list, metric1, metric2)
    system_level = compute_system_level_correlations(metrics_list, metric1, metric2)
    global_level = compute_global_correlations(metrics_list, metric1, metric2)
    results = {
        'summary_level': summary_level,
        'system_level': system_level,
        'global': global_level
    }
    return results


class CorrelateSubcommand(Subcommand):
    @overrides
    def add_subparser(self, parser: argparse._SubParsersAction):
        description = 'Calculate the correlation between two different metrics'
        self.parser = parser.add_parser('correlate', description=description, help=description)
        self.parser.add_argument(
            '--metrics-jsonl-files',
            nargs='+',
            help='The jsonl files with the metric values. If the values are split across multiple files, they can all '
                 'be passed as arguments.',
            required=True
        )
        self.parser.add_argument(
            '--metrics',
            nargs=2,
            type=str,
            help='The flattened names of the two metrics that should be correlated',
            required=True
        )
        self.parser.add_argument(
            '--summarizer-type',
            choices=['all', 'reference', 'peer'],
            help='The type of summarizer which should be included in the correlation calculation',
            required=True
        )
        self.parser.add_argument(
            '--output-file',
            type=str,
            help='The json output file which will contain the final correlations'
        )
        self.parser.add_argument(
            '--log-file',
            type=str,
            help='The file where the log should be written'
        )
        self.parser.add_argument(
            '--silent',
            action='store_true',
            help='Controls whether the log should be written to stdout'
        )
        self.parser.set_defaults(func=self.run)

    def run(self, args):
        prepare_global_logging(file_path=args.log_file, silent=args.silent)

        metric1, metric2 = args.metrics
        results = compute_correlation(args.metrics_jsonl_files, metric1, metric2, args.summarizer_type)

        if args.output_file:
            dirname = os.path.dirname(args.output_file)
            if dirname:
                os.makedirs(dirname, exist_ok=True)
            with open(args.output_file, 'w') as out:
                out.write(json.dumps(results, indent=2))

        if not args.silent:
            logger.info(json.dumps(results, indent=2))
